@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2016 Codethink Limited
 #
@@ -29,6 +28,7 @@ from contextlib import ExitStack
 
 import psutil
 
+from .._exceptions import SandboxError
 from .. import utils, _signals
 from ._mount import MountMap
 from . import Sandbox, SandboxFlags
@@ -67,6 +67,11 @@ class SandboxBwrap(Sandbox):
         if env is None:
             env = self._get_environment()
 
+        if not self._has_command(command[0], env):
+            raise SandboxError("Staged artifacts do not provide command "
+                               "'{}'".format(command[0]),
+                               reason='missing-command')
+
         # We want command args as a list of strings
         if isinstance(command, str):
             command = [command]
@@ -207,7 +212,7 @@ class SandboxBwrap(Sandbox):
 
                 # Skip removal of directories which already existed before
                 # launching bwrap
-                if not existing_basedirs[basedir]:
+                if existing_basedirs[basedir]:
                     continue
 
                 base_directory = os.path.join(root_mount_source, basedir)
